<?php

class Controller_Api_V0_Teacher extends Controller_Handler_Api {
    private function normalizeTeacherData($teacherData) {
        if (!isset($teacherData->id) && isset($teacherData->externalID)) {
            $teacherData->id = Model_Teacher::withExternalID($teacherData->externalID);
        }

        if (!isset($teacherData->id) || ($teacherData->id == -1)) {
            $foundList = Model_Teachers::search([$teacherData->firstName, $teacherData->secondName, $teacherData->lastName], $teacherData->facultyID);
            if (count($foundList) == 1) {
                $teacherData->id = $foundList[0]['ID'];
            } elseif (count($foundList) > 1) {
                throw new InvalidArgumentException('Duplicate teacher');
            }
        }

        if (!isset($teacherData->departmentID)) {
            if (isset($teacherData->departmentExternalID)) {
                $depid = $this->getDepIDByExternalID($teacherData->departmentExternalID);
                // !!! TODO: implement department synchronization
//                if ( $depid === NULL ) {
//                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка коде справочника подразделения.') );
//                }
                $teacherData->departmentID = $depid;
            } elseif (isset($teacherData->departmentName)) {
                $depid = $this->getDepIDByName($teacherData->departmentName);
//                if ( $depid === NULL ) {
//                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в имени подразделения.') );
//                }
                $teacherData->departmentID = $depid;
            } else {
                throw new InvalidArgumentException('Teacher has no department');
            }
        }

        if (!isset($teacherData->jobPositionID) && isset($teacherData->jobPositionName)) {
            $teacherData->jobPositionID = Model_Faculties::getJobPositionIdByName($teacherData->jobPositionName);
            if (is_null($teacherData->jobPositionID)) {
                $teacherData->jobPositionID = Model_Faculties::createJobPosition($teacherData->jobPositionName);
            }
        } elseif (!isset($teacherData->jobPositionName)) {
            throw new InvalidArgumentException('Teacher has no job position');
        }

        switch ($teacherData->status) {
            case 'Работает':
                $teacherData->status = true;
                break;

            case 'Уволен':
                $teacherData->status = false;
                break;

            default:
                throw new InvalidArgumentException('Teacher has bad status');
        }

        return $teacherData;
    }

    private function createTeacher($teacherData) {
        $teacher = Model_Teacher::make()
            ->faculty($teacherData->facultyID)
            ->jobPosition($teacherData->jobPositionID)
            ->firstName($teacherData->firstName)
            ->secondName($teacherData->secondName)
            ->lastName($teacherData->lastName);
        if (isset($teacherData->departmentID)) {
            $teacher->department($teacherData->departmentID);
        }
        if (isset($teacherData->externalID)) {
            $teacher = $teacher->externalID($teacherData->externalID);
        }
        if (isset($teacherData->inila)) {
            $teacher = $teacher->inila($teacherData->inila);
        }
        $teacher = $teacher->create();

        return (int)$teacher->ID;
    }

    private function updateTeacher($teacherData) {
        $teacher = Model_Teacher::with($teacherData->id);
        $teacher->changeInfo($teacherData->externalID, $teacherData->inila,
            $teacherData->lastName, $teacherData->firstName, $teacherData->secondName,
            $teacherData->jobPositionID, $teacherData->departmentID, $teacherData->status
        );

        return (int)$teacher->ID;
    }

//    /**
//     * @api {put} api/v0/teacher Add new teacher(s)
//     * @apiName Add new teacher
//     * @apiGroup Teacher
//     * @apiVersion 0.1.0
//     * @apiParam {String} token Api-токен
//     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
//     * @apiParam {String} lastName Фамилия преподавателя
//     * @apiParam {String} firstName Имя преподавателя
//     * @apiParam {String} [secondName] Отчество преподавателя
//     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
//     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
//     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
//     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
//     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
//     * @apiParam {String=Уволен,Работает} status Статус преподавателя
//     */
//    public function action_put_index() {
//        $res = [];
//        try {
//            if ( $this->request->query('batch') !== NULL ) {
//                $data = json_decode($this->request->body());
//                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);
//
//                foreach ($data->teachers as $item) {
//                    try
//                    {
//                        $item->facultyID = $facultyID;
//                        $teacherData = $this->normalizeTeacherData($item);
//
//                        if ( isset($teacherData) && $teacherData->status ) {
//                            if ( isset($teacherData->id)) {
//                                $res[] = $this->updateTeacher($teacherData);
//                            }
//                            else {
//                                $res[] = $this->createTeacher($teacherData);
//                            }
//                        }
//                    }
//                    catch (Exception  $e)
//                    {
//                        echo 'Yet another exception:';
//                        print_r($e->getMessage());
//                    }
//                }
//            } else {
//                $data = [ 'firstName' => $this->request->query('firstName')
//                    , 'secondName' => $this->request->query('secondName')
//                    , 'lastName' => $this->request->query('lastName')
//                    , 'departmentName' => $this->request->query('departmentName')
//                    , 'departmentID' => $this->request->query('departmentID')
//                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
//                    , 'jobPositionName' => $this->request->query('jobPositionName')
//                    , 'jobPositionID' => $this->request->query('jobPositionID')
//                    , 'status' => $this->request->query('status')];
//
//                $data = $this->normalizeTeacherData((object)$data);
//                if ( $data->status ) {
//                    $res[] = $this->createTeacher($data);
//                }
//            }
//        } catch (Exception $e) {
//            $this->badRequestError($e->getMessage());
//        }
//
//        return $res;
//    }

//    /**
//     * @api {put} api/v0/teacher/WithID Update teacher(s) with particular inner ID
//     * @apiName Update teacher with inner id
//     * @apiGroup Teacher
//     * @apiVersion 0.1.1
//     * @apiParam {String} token Api-токен
//     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
//     * @apiParam {Int} id Идентификатор преподавателя
//     * @apiParam {String} lastName Фамилия преподавателя
//     * @apiParam {String} firstName Имя преподавателя
//     * @apiParam {String} [secondName] Отчество преподавателя
//     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
//     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
//     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
//     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
//     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
//     * @apiParam {String=Уволен,Работает} status Статус преподавателя
//     */
//    public function action_put_WithID() {
//        try {
//            if ( $this->request->query('batch') !== NULL ) {
//                $data = json_decode($this->request->body());
//
//                foreach ($data as $item) {
//                    $data = $this->normalizeTeacherData($item);
//                    $this->updateTeacher($data);
//                }
//            } else {
//                $data = [ 'id' => $this->request->query('id')
//                    , 'firstName' => $this->request->query('firstName')
//                    , 'secondName' => $this->request->query('secondName')
//                    , 'lastName' => $this->request->query('lastName')
//                    , 'departmentName' => $this->request->query('departmentName')
//                    , 'departmentID' => $this->request->query('departmentID')
//                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
//                    , 'jobPositionName' => $this->request->query('jobPositionName')
//                    , 'jobPositionID' => $this->request->query('jobPositionID')
//                    , 'status' => $this->request->query('status')];
//
//                $data = $this->normalizeTeacherData((object)$data);
//                $this->updateTeacher($data);
//            }
//        } catch (Exception $e) {
//            $this->badRequestError($e->getMessage());
//        }
//
//        return "Success";
//    }

//    /**
//     * @api {put} api/v0/teacher/WithExternalID Add or update teacher(s) with particular 1C ID
//     * @apiName Update teacher with external id
//     * @apiGroup Teacher
//     * @apiVersion 0.1.1
//     * @apiParam {String} token Api-токен
//     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
//     * @apiParam {Int}    externalID Код справочника для этого преподавателя из 1С:Университет
//     * @apiParam {String} lastName Фамилия преподавателя
//     * @apiParam {String} firstName Имя преподавателя
//     * @apiParam {String} [secondName] Отчество преподавателя
//     * @apiParam {String} [inila] Хэш СНИЛС преподавателя (для авторизации через OpenID.sfedu.ru)
//     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
//     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
//     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
//     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
//     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
//     * @apiParam {String=Уволен,Работает} status Статус преподавателя
//     */
    public function action_put_index() {
        $res = [];

        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());
                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

                foreach ($data->teachers as $teacherData) {
                    try {
                        $teacherData->facultyID = $facultyID;
                        $teacherData = $this->normalizeTeacherData($teacherData);

                        if (isset($teacherData->id)) {
                            $res[] = $this->updateTeacher($teacherData);
                        } else {
                            $res[] = $this->createTeacher($teacherData);
                        }
                    } catch (Exception $e) {
                        Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                            '{0}' => 'SYNC_TEACHERS',
                            '{1}' => $e->getMessage(),
                            '{2}' => json_encode($teacherData),
                        ));
                    }
                }
            } else {
//                $data = ['externalID' => $this->request->query('externalID')
//                    , 'firstName' => $this->request->query('firstName')
//                    , 'secondName' => $this->request->query('secondName')
//                    , 'lastName' => $this->request->query('lastName')
//                    , 'inila' => $this->request->query('inila')
//                    , 'departmentName' => $this->request->query('departmentName')
//                    , 'departmentID' => $this->request->query('departmentID')
//                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
//                    , 'jobPositionName' => $this->request->query('jobPositionName')
//                    , 'jobPositionID' => $this->request->query('jobPositionID')
//                    , 'status' => $this->request->query('status')];
//
//                $data = $this->normalizeTeacherData((object)$data);
//                if ($data->id == -1) {
//                    if ($data->status) {
//                        $this->createTeacher($data);
//                    }
//                } else {
//                    $this->updateTeacher($data);
//                }
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
